-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio               "EskaQuestTracker.Classes.QuestBlock"                   ""
-- ========================================================================== --
namespace "EQT"
-- ========================================================================== --
class "QuestBlock" inherit "Block"
  -- ======================================================================== --
  -- Methods
  -- ======================================================================== --
  __Arguments__{ Quest }
  function AddQuest(self, quest)
    if not self.quests:Contains(quest) then
      local header = self:GetHeader(quest.header)
      if not header then
        header = self:NewHeader(quest.header)
      end
      header:AddQuest(quest)
      self.quests:Insert(quest)
    end
  end

  __Arguments__ { Number }
  function RemoveQuest(self, questID)
    local quest = self:GetQuest(questID)
    if quest then
      self:RemoveQuest(quest)
    end
  end

  __Arguments__{ Quest }
  function RemoveQuest(self, quest)
    self.quests:Remove(quest)
    local header = self:GetHeader(quest.header)
    if header then
      header:RemoveQuest(quest)
      if header:GetQuestNum() == 0 then
        self:RemoveHeader(quest.header)
        header.isReusable = true
      end
    end
    quest.isReusable = true
  end

  __Arguments__ { Number }
  function GetQuest(self, questID)
    for _, quest in self.quests:GetIterator() do
      if quest.id == questID then
        return quest
      end
    end
  end

  __Arguments__ { String }
  function GetHeader(self, name)
    return self.headers[name]
  end

  __Arguments__ { String }
  function NewHeader(self, name)
    local header = _ObjectManager:GetQuestHeader()
    header.name = name
    header:SetParent(self.frame)

    self.headers[name] = header

    return header
  end

  __Arguments__ { String }
  function RemoveHeader(self, name)
    self.headers[name] = nil
  end

  __Arguments__()
  function Draw(self)
    local previousFrame
    local height = 0
    local isFirst = true

    for headerName, header in self.headers.Values:ToList():Sort("x,y=>x.nearestQuestDistance<y.nearestQuestDistance"):GetIterator() do
      header:ClearAllPoints()
      header:Show()
      header:Draw()

      if isFirst then
        header.frame:SetPoint("TOPLEFT", 0, -35)
        header.frame:SetPoint("TOPRIGHT", 0, -35)
        isFirst = false
      else
        header.frame:SetPoint("TOPLEFT", previousFrame, "BOTTOMLEFT", 0, -5)
        header.frame:SetPoint("TOPRIGHT", previousFrame, "BOTTOMRIGHT")
      end
        height = height + header.height
        previousFrame = header.frame
    end
    self.height = self.baseHeight + height
  end

  __Static__() property "showOnlyQuestsOnZone" {
    TYPE = Boolean,
    SET = function(self, filteringByZone)
      _DB.Quests.filteringByZone = filteringByZone
    end,
    GET = function(self) return _DB.Quests.filteringByZone end
  }

  __Static__() property "useSortMethods" {
    TYPE = Boolean,
    GET = function(self) return _DB.Quest.sortByDistance end
  }

  __Static__() property "sortByDistance" {
    SET = function(self, sortedByDistance)
      _DB.Quests.sortByDistance = sortByDistance
    end,
    GET = function(self) return _DB.Quests.sortByDistance end
  }

  -- ======================================================================== --
  -- Contructor
  -- ======================================================================== --
  function QuestBlock(self)
    Super(self, "Quests", 20)
    self.text = "Quests"

    self.quests = ObjectArray(Quest)
    self.headers = Dictionary()
  end


endclass "QuestBlock"
